@@ -5,7 +5,7 @@ use crate::{
     },
     constants::{Type, Class},
     records::{data::RData, RecordSet},
-    Error, Result
+    Result
 };
 
 {% if async == "true" -%}
@@ -27,17 +27,9 @@ pub struct Client {
 
 impl Client {
     /// Creates a new instance of [`Client`] with specified configuration.
-    ///
-    /// # Returns
-    ///
-    /// [`NoNameservers`] - if no nameserver is specified in the configuration
-    ///
-    /// [`NoNameservers`]: Error::NoNameservers
     #[inline(always)]
     pub {{ as }} fn new(conf: ClientConfig) -> Result<Self> {
-        if !conf.has_nameserver() {
-            return Err(Error::NoNameservers);
-        }
+        conf.check()?;
         Ok(Self {
             internal: ClientImpl::new(conf){{ aw }}?,
         })
@@ -54,11 +46,11 @@ impl Client {
     /// This method gives the control over buffer management to the caller.
     /// The response message is written into `buf` and its length is returned in the result.
     ///
-    /// The received response can be parsed using [`MessageIterator`].
+    /// See the [`message::reader`] module for ways to parse the received response.
     ///
     /// This method doesn't allocate.
     ///
-    /// [`MessageIterator`]: crate::message::reader::MessageIterator
+    /// [`message::reader`]: crate::message::reader
     #[inline(always)]
     pub {{ as }} fn query_raw(&mut self, qname: &str, qtype: Type, qclass: Class, buf: &mut [u8]) -> Result<usize> {
         self.internal.query_raw(qname, qtype, qclass, buf){{ aw }}
